"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.properties.ListSize import (
    ListSize,  # pylint: disable=E0401
)


class TestListSize(BaseRuleTestCase):
    """Test List Size Property Configuration"""

    def setUp(self):
        """Setup"""
        super(TestListSize, self).setUp()
        self.collection.register(ListSize())

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative_iam_managed_policies(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources_iam_managedpolicyarns.yaml", 3
        )

    def test_file_negative_iam_user_groups(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources_iam_user_groups.yaml", 1
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.rds.InstanceSize import (
    InstanceSize,  # pylint: disable=E0401
)


class TestInstanceSize(BaseRuleTestCase):
    """Test RDS Instance Size Configuration"""

    def setUp(self):
        """Setup"""
        super(TestInstanceSize, self).setUp()
        self.collection.register(InstanceSize())
        self.success_templates = [
            "test/fixtures/templates/good/resources/rds/instance_sizes.yaml"
        ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative_alias(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/rds/instance_sizes.yaml",
            7,
            ["us-east-1", "eu-west-3"],
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import json
from test.unit.rules import BaseRuleTestCase

import cfnlint.core
import cfnlint.helpers
from cfnlint.rules.resources.properties.Properties import (
    Properties,  # pylint: disable=E0401
)


class TestResourceProperties(BaseRuleTestCase):
    """Test Resource Properties"""

    def setUp(self):
        """Setup"""
        super(TestResourceProperties, self).setUp()
        self.collection.register(Properties())
        self.success_templates = [
            "test/fixtures/templates/good/resource_properties.yaml",
            "test/fixtures/templates/good/resources/properties/templated_code.yaml",
            "test/fixtures/templates/good/resources/properties/properties_nested_if.yaml",
        ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative(self):
        """Test failure"""
        self.helper_file_negative("test/fixtures/templates/bad/generic.yaml", 9)

    def test_file_negative_2(self):
        """Failure test"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/object_should_be_list.yaml", 5
        )

    def test_file_negative_3(self):
        """Failure test"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resource_properties.yaml", 8
        )

    def test_E3012_in_bad_template(self):
        """Test E3012 in known-bad template"""
        filename = "test/fixtures/templates/bad/resource_properties.yaml"
        (args, _, _) = cfnlint.core.get_args_filenames(["--template", filename])
        (template, rules, _) = cfnlint.core.get_template_rules(filename, args)
        results = cfnlint.core.run_checks(filename, template, rules, ["us-east-1"])
        matched_rule_ids = [r.rule.id for r in results]
        self.assertIn("E3012", matched_rule_ids)

    def test_E3012_match_has_extra_attributes(self):
        """Test E3012 in has custom attributes"""
        filename = "test/fixtures/templates/bad/resource_properties.yaml"
        (args, _, _) = cfnlint.core.get_args_filenames(["--template", filename])
        (template, rules, _) = cfnlint.core.get_template_rules(filename, args)
        results = cfnlint.core.run_checks(filename, template, rules, ["us-east-1"])
        custom_attrs = ["actual_type", "expected_type"]
        for r in results:
            if r.rule.id == "E3012":
                for ca in custom_attrs:
                    assert hasattr(r, ca), "Attribute {} was not found".format(ca)


class TestSpecifiedCustomResourceProperties(TestResourceProperties):
    """Repeat Resource Properties tests with Custom Resource override spec provided"""

    def setUp(self):
        """Setup"""
        super(TestSpecifiedCustomResourceProperties, self).setUp()
        # Add a Spec override that specifies the Custom::SpecifiedCustomResource type
        with open("test/fixtures/templates/override_spec/custom.json") as fp:
            custom_spec = json.load(fp)
        cfnlint.helpers.set_specs(custom_spec)
        # Reset Spec override after test
        self.addCleanup(cfnlint.helpers.initialize_specs)

    # ... all TestResourceProperties test cases are re-run with override spec ...

    def test_file_negative_custom(self):
        """Additional failure test for specified Custom Resource validation"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/properties/custom.yaml", 2
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.functions.FindInMap import FindInMap  # pylint: disable=E0401


class TestRulesFindInMap(BaseRuleTestCase):
    """Test Rules Get Att"""

    def setUp(self):
        """Setup"""
        super(TestRulesFindInMap, self).setUp()
        self.collection.register(FindInMap())
        self.success_templates = [
            "test/fixtures/templates/good/functions_findinmap.yaml",
            "test/fixtures/templates/good/functions_findinmap_enhanced.yaml",
            "test/fixtures/templates/good/functions_findinmap_default_value.yaml",
        ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/functions_findinmap.yaml", 7
        )

    def test_file_negative_enhanced(self):
        self.helper_file_negative(
            "test/fixtures/templates/bad/functions_findinmap_default_value.yaml", 15
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.UpdateReplacePolicyDeletionPolicyOnStatefulResourceTypes import (
    UpdateReplacePolicyDeletionPolicyOnStatefulResourceTypes,  # pylint: disable=E0401
)


class TestUpdateReplacePolicyDeletionPolicyOnStatefulResourceTypes(BaseRuleTestCase):
    """Test StateMachine for Step Functions"""

    def setUp(self):
        """Setup"""
        super(
            TestUpdateReplacePolicyDeletionPolicyOnStatefulResourceTypes, self
        ).setUp()
        self.collection.register(
            UpdateReplacePolicyDeletionPolicyOnStatefulResourceTypes()
        )
        self.success_templates = [
            "test/fixtures/templates/good/resources/dynamodb/delete_policy.yaml"
        ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative_alias(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/dynamodb/delete_policy.yaml", 1
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

import regex as re

from cfnlint.data import CloudSpecs
from cfnlint.helpers import RESOURCE_SPECS, load_resource
from cfnlint.rules.resources.properties.AllowedPattern import (
    AllowedPattern,  # pylint: disable=E0401
)


class TestAllowedPattern(BaseRuleTestCase):
    """Test Allowed Value Property Configuration"""

    def setUp(self):
        """Setup"""
        super(TestAllowedPattern, self).setUp()
        self.collection.register(AllowedPattern())
        self.success_templates = [
            "test/fixtures/templates/good/resources/properties/allowed_pattern.yaml"
        ]

        self.spec = RESOURCE_SPECS["us-east-1"]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_template_config(self):
        """Test strict false"""
        self.helper_file_rule_config(
            "test/fixtures/templates/bad/properties_sg_ingress.yaml",
            {"exceptions": ["Special charaters*"]},
            1,
        )

    def test_file_negative_sg_ingress(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/properties_sg_ingress.yaml", 2
        )

    def test_valid_regex(self):
        """Test Resource Type Value Regex"""
        for r_name, r_values in self.spec.get("ValueTypes").items():
            if r_values.get("AllowedPatternRegex"):
                p_regex = r_values.get("AllowedPatternRegex")
                try:
                    re.compile(p_regex)
                except re.error:
                    self.fail(
                        "Invalid regex value %s specified for ValueType %s"
                        % (p_regex, r_name)
                    )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.properties.ValuePrimitiveType import (
    ValuePrimitiveType,  # pylint: disable=E0401
)


class TestResourceValuePrimitiveType(BaseRuleTestCase):
    """Test Primitive Value Types"""

    def setUp(self):
        """Setup"""
        super(TestResourceValuePrimitiveType, self).setUp()
        self.collection.register(ValuePrimitiveType())

    success_templates = [
        "test/fixtures/templates/good/generic.yaml",
        "test/fixtures/templates/good/resource_properties.yaml",
        "test/fixtures/templates/good/resources/properties/primitive_types.yaml",
    ]

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_bad_null_values(self):
        """Test strict false"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/properties/primitive_types.yaml", 1
        )

    def test_template_config(self):
        """Test strict false"""
        self.helper_file_rule_config(
            "test/fixtures/templates/good/resources/properties/value_non_strict.yaml",
            {"strict": False},
            0,
        )
        self.helper_file_rule_config(
            "test/fixtures/templates/bad/resources/properties/value_non_strict.yaml",
            {"strict": False},
            4,
        )

    def test_bad_map_values(self):
        """Test strict false"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/properties/primitive_types_map.yaml",
            2,
        )

    def test_file_negative_nist_high_main(self):
        """Generic Test failure"""
        self.helper_file_rule_config(
            "test/fixtures/templates/quickstart/nist_high_main.yaml",
            {"strict": True},
            6,
        )

    def test_file_negative_nist_high_app(self):
        """Generic Test failure"""
        self.helper_file_rule_config(
            "test/fixtures/templates/quickstart/nist_application.yaml",
            {"strict": True},
            53,
        )

    def test_file_negative_nist_config_rules(self):
        """Generic Test failure"""
        self.helper_file_rule_config(
            "test/fixtures/templates/quickstart/nist_config_rules.yaml",
            {"strict": True},
            2,
        )

    def test_file_negative_generic(self):
        """Generic Test failure"""
        self.helper_file_rule_config(
            "test/fixtures/templates/bad/generic.yaml", {"strict": True}, 7
        )


class TestResourceValuePrimitiveTypeNonStrict(BaseRuleTestCase):
    """Test Primitive Value Types"""

    def setUp(self):
        """Setup"""
        self.rule = ValuePrimitiveType()
        self.rule.config["strict"] = False

    def test_file_positive(self):
        """Test Positive"""
        # Test Booleans
        self.assertEqual(
            len(self.rule._value_check("True", ["test"], "Boolean", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check("False", ["test"], "Boolean", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check(1, ["test"], "Boolean", False, {})), 1
        )
        # Test Strings
        self.assertEqual(
            len(self.rule._value_check(1, ["test"], "String", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check(2, ["test"], "String", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check(True, ["test"], "String", False, {})), 0
        )
        # Test Integer
        self.assertEqual(
            len(self.rule._value_check("1", ["test"], "Integer", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check("1.2", ["test"], "Integer", False, {})), 1
        )
        self.assertEqual(
            len(self.rule._value_check(True, ["test"], "Integer", False, {})), 1
        )
        self.assertEqual(
            len(self.rule._value_check("test", ["test"], "Integer", False, {})), 1
        )
        # Test Double
        self.assertEqual(
            len(self.rule._value_check("1", ["test"], "Double", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check("1.2", ["test"], "Double", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check(1, ["test"], "Double", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check(True, ["test"], "Double", False, {})), 1
        )
        self.assertEqual(
            len(self.rule._value_check("test", ["test"], "Double", False, {})), 1
        )
        # Test Long
        self.assertEqual(
            len(
                self.rule._value_check(str(65536 * 65536), ["test"], "Long", False, {})
            ),
            0,
        )
        self.assertEqual(
            len(self.rule._value_check("1", ["test"], "Long", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check("1.2", ["test"], "Long", False, {})), 1
        )
        self.assertEqual(
            len(self.rule._value_check(1.2, ["test"], "Long", False, {})), 1
        )
        self.assertEqual(
            len(self.rule._value_check(65536 * 65536, ["test"], "Long", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check(True, ["test"], "Long", False, {})), 1
        )
        self.assertEqual(
            len(self.rule._value_check("test", ["test"], "Long", False, {})), 1
        )
        # Test Unknown type doesn't return error
        self.assertEqual(
            len(self.rule._value_check(1, ["test"], "Unknown", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check("1", ["test"], "Unknown", False, {})), 0
        )
        self.assertEqual(
            len(self.rule._value_check(True, ["test"], "Unknown", False, {})), 0
        )


class TestResourceValuePrimitiveTypeCheckValue(BaseRuleTestCase):
    """Test Check Value for maps"""

    def setUp(self):
        """Setup"""
        self.rule = ValuePrimitiveType()
        self.rule.config["strict"] = False

    def test_file_check_value_good_function(self):
        results = self.rule.check_value(
            {"key": {"Ref": ["Parameter"]}},
            [],
            primitive_type="String",
            item_type="Map",
        )
        self.assertEqual(len(results), 0)

    def test_file_check_value_is_string(self):
        results = self.rule.check_value(
            {"key": 1}, [], primitive_type="Integer", item_type="Map"
        )
        self.assertEqual(len(results), 0)

    def test_file_check_value_is_json(self):
        results = self.rule.check_value(
            {"key": {}}, [], primitive_type="Json", item_type="Map"
        )
        self.assertEqual(len(results), 0)

    def test_file_check_value_bad_function(self):
        results = self.rule.check_value(
            {"key": {"Func": ["Parameter"]}},
            [],
            primitive_type="Boolean",
            item_type="Map",
        )
        self.assertEqual(len(results), 1)
        self.assertEqual(
            results[0].message,
            "Use a valid function [Fn::Base64, Fn::Cidr, Fn::Contains, "
            "Fn::FindInMap, Fn::GetAtt, Fn::If, Fn::ImportValue, Fn::Join, Fn::Length, "
            "Fn::Select, Fn::Sub, Fn::ToJsonString, Ref] when "
            "providing a value of type [Boolean]",
        )

    def test_file_check_value_bad_object(self):
        results = self.rule.check_value(
            {"key": {"Func": ["Parameter"], "Func2": ["Parameter"]}},
            [],
            primitive_type="String",
            item_type="Map",
        )
        self.assertEqual(len(results), 1)
        self.assertEqual(
            results[0].message,
            "Use a valid function [Fn::Base64, Fn::Cidr, Fn::Contains, "
            "Fn::FindInMap, Fn::GetAtt, Fn::If, Fn::ImportValue, Fn::Join, Fn::Length, "
            "Fn::Select, Fn::Sub, Fn::ToJsonString, Ref] when "
            "providing a value of type [String]",
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.testlib.testcase import BaseTestCase

import cfnlint.decode.node  # pylint: disable=E0401


class TestNode(BaseTestCase):
    """Test Node Objects"""

    def test_success_init(self):
        """Test Dict Object"""
        template = cfnlint.decode.node.dict_node({"test": "string"}, (0, 1), (2, 3))

        self.assertEqual(template, {"test": "string"})
        self.assertEqual(template.start_mark[0], 0)
        self.assertEqual(template.start_mark[1], 1)
        self.assertEqual(template.end_mark[0], 2)
        self.assertEqual(template.end_mark[1], 3)

    def test_success_fnif_object(self):
        """Test Dict Object"""
        template = cfnlint.decode.node.dict_node(
            {
                "Fn::If": [
                    "string",
                    cfnlint.decode.node.dict_node({"Test": True}, (0, 1), (2, 3)),
                    cfnlint.decode.node.dict_node({"Test": False}, (0, 1), (2, 3)),
                ]
            },
            (0, 1),
            (2, 3),
        )

        results = []
        for items, p in template.items_safe():
            results.append((items, p))

        self.assertEqual(
            results, [({"Test": True}, ["Fn::If", 1]), ({"Test": False}, ["Fn::If", 2])]
        )

    def test_success_fnif_string(self):
        """Test Dict Object"""
        template = cfnlint.decode.node.dict_node(
            {"Fn::If": ["string", "True", "False"]}, (0, 1), (2, 3)
        )

        results = []
        for items, p in template.items_safe():
            results.append((items, p))

        self.assertEqual(results, [("True", ["Fn::If", 1]), ("False", ["Fn::If", 2])])

    def test_success_fnif_list(self):
        """Test List Object"""
        template = cfnlint.decode.node.dict_node(
            {"Fn::If": ["string", ["1", "2", "3"], ["a", "b", "c"]]}, (0, 1), (2, 3)
        )

        results = []
        for items, p in template.items_safe():
            results.append((items, p))

        self.assertEqual(
            results,
            [(["1", "2", "3"], ["Fn::If", 1]), (["a", "b", "c"], ["Fn::If", 2])],
        )

    def test_success_fnif_ref_novalue(self):
        """Test List Object"""
        template = cfnlint.decode.node.dict_node(
            {
                "Fn::If": [
                    "string",
                    True,
                    cfnlint.decode.node.dict_node(
                        {"Ref": "AWS::NoValue"}, (0, 1), (2, 3)
                    ),
                ]
            },
            (0, 1),
            (2, 3),
        )

        results = []
        for items, p in template.items_safe():
            results.append((items, p))

        self.assertEqual(results, [(True, ["Fn::If", 1])])

    def test_success_fnif_nested(self):
        """Test List Object"""
        template = cfnlint.decode.node.dict_node(
            {
                "Fn::If": [
                    "string",
                    cfnlint.decode.node.dict_node({"Test": "True"}, (0, 1), (2, 3)),
                    cfnlint.decode.node.dict_node(
                        {
                            "Fn::If": [
                                "nested",
                                cfnlint.decode.node.dict_node(
                                    {"NestedTest": "False,True"}, (0, 1), (2, 3)
                                ),
                                False,
                            ]
                        },
                        (0, 1),
                        (2, 3),
                    ),
                ]
            },
            (0, 1),
            (2, 3),
        )

        results = []
        for items, p in template.items_safe(path=["Start"]):
            results.append((items, p))

        # Adding testing of Paths to make sure nested paths are good
        self.assertEqual(
            results,
            [
                ({"Test": "True"}, ["Start", "Fn::If", 1]),
                ({"NestedTest": "False,True"}, ["Start", "Fn::If", 2, "Fn::If", 1]),
                (False, ["Start", "Fn::If", 2, "Fn::If", 2]),
            ],
        )

        # Testing the filters based on type
        results = []
        for items, p in template.items_safe(type_t=(dict)):
            results.append((items, p))

        self.assertEqual(
            results,
            [
                ({"Test": "True"}, ["Fn::If", 1]),
                ({"NestedTest": "False,True"}, ["Fn::If", 2, "Fn::If", 1]),
            ],
        )

    def test_success_fnif_get(self):
        """Test Dict Object"""
        template = cfnlint.decode.node.dict_node(
            {
                "Test": cfnlint.decode.node.dict_node(
                    {"Fn::If": ["string", "True", "False"]}, (0, 1), (2, 3)
                )
            },
            (0, 1),
            (2, 3),
        )

        obj = template.get_safe("Test")
        self.assertEqual(
            obj, [("True", ["Test", "Fn::If", 1]), ("False", ["Test", "Fn::If", 2])]
        )

    def test_success_fnif_get_nested(self):
        """Test Dict Object"""
        template = cfnlint.decode.node.dict_node(
            {
                "Test": cfnlint.decode.node.dict_node(
                    {
                        "Fn::If": [
                            "string",
                            cfnlint.decode.node.dict_node(
                                {"Fn::If": ["string", "True,True", "True,False"]},
                                (0, 1),
                                (2, 3),
                            ),
                            "False",
                        ]
                    },
                    (0, 1),
                    (2, 3),
                )
            },
            (0, 1),
            (2, 3),
        )

        obj = template.get_safe("Test")
        self.assertEqual(
            obj,
            [
                ("True,True", ["Test", "Fn::If", 1, "Fn::If", 1]),
                ("True,False", ["Test", "Fn::If", 1, "Fn::If", 2]),
                ("False", ["Test", "Fn::If", 2]),
            ],
        )

    def test_success_fnif_list_strings(self):
        """Test List Object"""
        template = cfnlint.decode.node.list_node(["1", "2", "3"], (0, 1), (2, 3))

        results = []
        for v, p in template.items_safe():
            results.append((v, p))

        self.assertEqual(results, [("1", [0]), ("2", [1]), ("3", [2])])

    def test_success_fnif_list_conditions(self):
        """Test List Object"""
        template = cfnlint.decode.node.list_node(
            [
                "1",
                "2",
                cfnlint.decode.node.dict_node(
                    {"Fn::If": ["string", "4", "5"]}, (0, 1), (2, 3)
                ),
            ],
            (0, 1),
            (2, 3),
        )

        results = []
        for v, p in template.items_safe():
            results.append((v, p))

        self.assertEqual(
            results,
            [("1", [0]), ("2", [1]), ("4", [2, "Fn::If", 1]), ("5", [2, "Fn::If", 2])],
        )

    def test_success_fnif_list_conditions_no_value(self):
        """Test List Object"""
        template = cfnlint.decode.node.list_node(
            [
                "1",
                "2",
                cfnlint.decode.node.dict_node(
                    {
                        "Fn::If": [
                            "string",
                            "4",
                            cfnlint.decode.node.dict_node(
                                {"Ref": "AWS::NoValue"}, (0, 1), (2, 3)
                            ),
                        ]
                    },
                    (0, 1),
                    (2, 3),
                ),
            ],
            (0, 1),
            (2, 3),
        )

        results = []
        for v, p in template.items_safe():
            results.append((v, p))

        self.assertEqual(results, [("1", [0]), ("2", [1]), ("4", [2, "Fn::If", 1])])

    def test_success_dict_select(self):
        """Test if select could return an object"""

        template = cfnlint.decode.node.dict_node(
            {
                "Fn::Select": cfnlint.decode.node.list_node(
                    [
                        0,
                        cfnlint.decode.node.dict_node(
                            {"Fn::FindInMap": ["mapping", "key", "value"]},
                            (0, 1),
                            (2, 3),
                        ),
                    ],
                    (0, 1),
                    (2, 3),
                )
            },
            (0, 1),
            (2, 3),
        )

        self.assertTrue(template.is_function_returning_object())

    def test_failure_dict_select(self):
        """Test if select could return an object"""

        template = cfnlint.decode.node.dict_node(
            {
                "Key": "Value",
                "Fn::Select": cfnlint.decode.node.list_node(
                    [
                        0,
                        cfnlint.decode.node.dict_node(
                            {"Fn::FindInMap": ["mapping", "key", "value"]},
                            (0, 1),
                            (2, 3),
                        ),
                    ],
                    (0, 1),
                    (2, 3),
                ),
            },
            (0, 1),
            (2, 3),
        )

        self.assertFalse(template.is_function_returning_object())

        template = cfnlint.decode.node.dict_node(
            {
                "Key": "Value",
                "Fn::Select": cfnlint.decode.node.list_node(
                    [0, cfnlint.decode.node.list_node([0, 1, 2], (0, 1), (2, 3))],
                    (0, 1),
                    (2, 3),
                ),
            },
            (0, 1),
            (2, 3),
        )

        self.assertFalse(template.is_function_returning_object())



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.testlib.testcase import BaseTestCase

import cfnlint.config
from cfnlint.rules import RulesCollection
from cfnlint.runner import Runner


class BaseRuleTestCase(BaseTestCase):
    """Used for Testing Rules"""

    success_templates = []

    def setUp(self):
        """Setup"""
        self.collection = RulesCollection(
            include_rules=["I"], include_experimental=True
        )

    def helper_file_positive(self):
        """Success test"""
        for filename in self.success_templates:
            template = self.load_template(filename)
            good_runner = Runner(self.collection, filename, template, ["us-east-1"], [])
            good_runner.transform()
            failures = good_runner.run()
            assert [] == failures, "Got failures {} on {}".format(failures, filename)

    def helper_file_rule_config(self, filename, config, err_count):
        """Success test with rule config included"""
        template = self.load_template(filename)
        rule_id = list(self.collection.rules.keys())[0]
        self.collection.rules[rule_id].configure(config)
        good_runner = Runner(self.collection, filename, template, ["us-east-1"], [])
        good_runner.transform()
        failures = good_runner.run()
        self.assertEqual(
            err_count,
            len(failures),
            "Expected {} failures but got {} on {}".format(
                err_count, failures, filename
            ),
        )
        self.collection.rules[rule_id].configure(config)

    def helper_file_positive_template(self, filename):
        """Success test with template parameter"""
        template = self.load_template(filename)
        good_runner = Runner(self.collection, filename, template, ["us-east-1"], [])
        good_runner.transform()
        self.assertEqual([], good_runner.run())

    def helper_file_negative(self, filename, err_count, regions=None):
        """Failure test"""
        regions = regions or ["us-east-1"]
        template = self.load_template(filename)
        bad_runner = Runner(self.collection, filename, template, regions, [])
        bad_runner.transform()
        errs = bad_runner.run()
        self.assertEqual(err_count, len(errs))



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.testlib.testcase import BaseTestCase

from cfnlint import Transform
from cfnlint.decode import cfn_yaml


class TestTransform(BaseTestCase):
    """Test Transform Parsing"""

    def test_parameter_for_autopublish_version(self):
        """Test Parameter is created for autopublish version run"""
        filename = "test/fixtures/templates/good/transform/auto_publish_alias.yaml"
        region = "us-east-1"
        template = cfn_yaml.load(filename)
        transformed_template = Transform(filename, template, region)
        transformed_template.transform_template()
        self.assertDictEqual(
            transformed_template._parameters, {"Stage1": "Alias", "Stage2": "Alias"}
        )
        self.assertDictEqual(
            transformed_template._template.get("Resources")
            .get("SkillFunctionAliasAlias")
            .get("Properties"),
            {
                "Name": "Alias",
                "FunctionName": {"Ref": "SkillFunction"},
                "FunctionVersion": {
                    "Fn::GetAtt": ["SkillFunctionVersion55ff35af87", "Version"]
                },
            },
        )

    def test_conversion_of_step_function_definition_uri(self):
        """Tests that the a serverless step function can convert a local path to a s3 path"""
        filename = (
            "test/fixtures/templates/good/transform/step_function_local_definition.yaml"
        )
        region = "us-east-1"
        template = cfn_yaml.load(filename)
        transformed_template = Transform(filename, template, region)
        transformed_template.transform_template()
        self.assertDictEqual(
            transformed_template._template.get("Resources")
            .get("StateMachine")
            .get("Properties")
            .get("DefinitionS3Location"),
            {"Bucket": "bucket", "Key": "value"},
        )

    def test_conversion_of_application_location(self):
        """Tests if serverless application converts location to string when dict"""
        filename = "test/fixtures/templates/good/transform/applications_location.yaml"
        region = "us-east-1"
        template = cfn_yaml.load(filename)
        transformed_template = Transform(filename, template, region)
        transformed_template.transform_template()
        self.assertEqual(
            transformed_template._template.get("Resources")
            .get("App1")
            .get("Properties")
            .get("TemplateURL"),
            "./step_function_local_definition.yaml",
        )
        self.assertEqual(
            transformed_template._template.get("Resources")
            .get("App2")
            .get("Properties")
            .get("TemplateURL"),
            "s3://bucket/value",
        )

    def test_conversion_of_step_function_definition_uri(self):
        """Tests that the a serverless step function can convert a local path to a s3 path"""
        filename = (
            "test/fixtures/templates/good/transform/step_function_local_definition.yaml"
        )
        region = "us-east-1"
        template = cfn_yaml.load(filename)
        transformed_template = Transform(filename, template, region)
        transformed_template.transform_template()
        self.assertDictEqual(
            transformed_template._template.get("Resources")
            .get("StateMachine")
            .get("Properties")
            .get("DefinitionS3Location"),
            {"Bucket": "bucket", "Key": "value"},
        )

    def test_conversion_of_serverless_function_uri(self):
        """Tests that the a serverless function can convert a CodeUri Sub"""
        filename = "test/fixtures/templates/good/transform/function_use_s3_uri.yaml"
        region = "us-east-1"
        template = cfn_yaml.load(filename)
        transformed_template = Transform(filename, template, region)
        transformed_template.transform_template()
        self.assertDictEqual(
            transformed_template._template.get("Resources")
            .get("Function")
            .get("Properties")
            .get("Code"),
            {"S3Bucket": "bucket", "S3Key": "value"},
        )

    def test_parameter_for_autopublish_version_bad(self):
        """Test Parameter is created for autopublish version run"""
        filename = "test/fixtures/templates/bad/transform/auto_publish_alias.yaml"
        region = "us-east-1"
        template = cfn_yaml.load(filename)
        transformed_template = Transform(filename, template, region)
        transformed_template.transform_template()
        self.assertDictEqual(transformed_template._parameters, {})

    def test_test_function_using_image_good(self):
        """Test Parameter is created for autopublish version run"""
        filename = "test/fixtures/templates/good/transform/function_using_image.yaml"
        region = "us-east-1"
        template = cfn_yaml.load(filename)
        transformed_template = Transform(filename, template, region)
        transformed_template.transform_template()
        self.assertDictEqual(transformed_template._parameters, {})

    def test_sam_with_language_extension(self):
        """Test language extension"""
        filename = "test/fixtures/templates/good/transform/language_extension.yaml"
        region = "us-east-1"
        template = cfn_yaml.load(filename)
        transformed_template = Transform(filename, template, region)
        results = transformed_template.transform_template()
        self.assertEqual(results, [])



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.ecs.FargateDeploymentSchedulingStrategy import (
    FargateDeploymentSchedulingStrategy,
)


class TestFargateServiceSchedulingStrategy(BaseRuleTestCase):
    """
    Test that Fargate service has correct scheduling strategy (REPLICA)
    """

    def setUp(self):
        super(TestFargateServiceSchedulingStrategy, self).setUp()
        self.collection.register(FargateDeploymentSchedulingStrategy())
        self.success_templates = [
            "test/fixtures/templates/good/resources/ecs/test_fargate_scheduling_strategy.yaml"
        ]

    def test_file_positive(self):
        self.helper_file_positive()

    def test_file_negative(self):
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources/ecs/test_fargate_scheduling_strategy.yaml",
            1,
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import logging
import test.fixtures.specs
from test.testlib.testcase import BaseTestCase

import cfnlint.helpers
from cfnlint.maintenance import patch_spec

LOGGER = logging.getLogger("cfnlint.maintenance")
LOGGER.addHandler(logging.NullHandler())


class TestPatchJson(BaseTestCase):
    """Used for Testing Rules"""

    def setUp(self):
        """Setup"""
        region = "us-east-1"

        self.spec = cfnlint.helpers.load_resource(
            test.fixtures.specs, "%s.json" % region
        )

    def test_success_rds_dbcluster(self):
        """Success test"""
        patched = patch_spec(self.spec, "all")
        self.assertTrue(
            patched["PropertyTypes"][
                "AWS::CloudFront::Distribution.DistributionConfig"
            ]["Properties"]["DefaultCacheBehavior"]["Required"]
        )
        self.assertTrue(
            patched["PropertyTypes"][
                "AWS::CloudFront::Distribution.DistributionConfig"
            ]["Properties"]["Origins"]["Required"]
        )
        self.assertTrue(
            patched["PropertyTypes"]["AWS::Cognito::UserPool.SmsConfiguration"][
                "Properties"
            ]["ExternalId"]["Required"]
        )
        self.assertEqual(
            patched["ResourceTypes"]["AWS::ServiceDiscovery::Instance"]["Properties"][
                "InstanceAttributes"
            ]["Type"],
            "Map",
        )
        self.assertEqual(
            patched["ResourceTypes"]["AWS::ServiceDiscovery::Instance"]["Properties"][
                "InstanceAttributes"
            ]["PrimitiveItemType"],
            "String",
        )

    def test_success_sbd_domain_removed(self):
        """Success removal of SBD Domain form unsupported regions"""
        patched = patch_spec(self.spec, "us-east-2")
        self.assertNotIn("AWS::SDB::Domain", patched["ResourceTypes"])

    def test_failure_in_patch_parent(self):
        """
        Doesn't fail when a parent doesn't exist
        """
        spec = self.spec

    def test_failure_in_patch_move(self):
        """
        Doesn't fail when final key doesn't match
        """
        spec = self.spec



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.testlib.testcase import BaseTestCase

import cfnlint.core
import cfnlint.helpers  # pylint: disable=E0401


class TestRunChecks(BaseTestCase):
    """Test Run Checks"""

    def test_good_template(self):
        """Test success run"""

        filename = "test/fixtures/templates/good/generic.yaml"
        (args, filenames, _) = cfnlint.core.get_args_filenames(["--template", filename])

        results = []
        for filename in filenames:
            (template, rules, _) = cfnlint.core.get_template_rules(filename, args)
            results.extend(
                cfnlint.core.run_checks(filename, template, rules, ["us-east-1"])
            )

        assert results == []

    def test_bad_template(self):
        """Test bad template"""

        filename = "test/fixtures/templates/quickstart/nat-instance.json"
        (args, filenames, _) = cfnlint.core.get_args_filenames(["--template", filename])
        results = []
        for filename in filenames:
            (template, rules, _) = cfnlint.core.get_template_rules(filename, args)
            results.extend(
                cfnlint.core.run_checks(filename, template, rules, ["us-east-1"])
            )

        assert results[0].rule.id == "W2506"
        assert results[1].rule.id == "W2001"

    def test_bad_region(self):
        """Test bad region"""
        filename = "test/fixtures/templates/good/generic.yaml"
        (args, filenames, _) = cfnlint.core.get_args_filenames(["--template", filename])
        (template, rules, _) = cfnlint.core.get_template_rules(filename, args)
        err = None
        try:
            cfnlint.core.run_checks(filename, template, rules, ["not-a-region"])
        except cfnlint.core.InvalidRegionException as e:
            err = e
        assert type(err) == cfnlint.core.InvalidRegionException



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.testlib.testcase import BaseTestCase
from typing import Any, Dict

from cfnlint.decode import decode_str
from cfnlint.rules import CloudFormationLintRule  # pylint: disable=E0401
from cfnlint.rules import Match, RuleMatch, RulesCollection
from cfnlint.runner import Runner


class TestCloudFormationRuleChild(BaseTestCase):
    """Test CloudFormation Rule"""

    def test_child_rules(self):
        class TestRuleParent(CloudFormationLintRule):
            """Def Rule"""

            id = "E1000"
            shortdesc = "Test Rule"
            description = "Test Rule Description"
            source_url = "https://github.com/aws-cloudformation/cfn-python-lint/"
            tags = ["resources"]
            child_rules: Dict[str, Any] = {"E1001": None}

            def match(self, _):
                return self.child_rules["E1001"].failure()

        class TestRuleChild(CloudFormationLintRule):
            """Def Rule"""

            id = "E1001"
            shortdesc = "Test Rule"
            description = "Test Rule Description"
            source_url = "https://github.com/aws-cloudformation/cfn-python-lint/"
            tags = ["resources"]

            def failure(self):
                return [RuleMatch(["key"], "failure", rule=self)]

        rule_collection = RulesCollection()
        test_rule_parent = TestRuleParent()
        test_rule_child = TestRuleChild()
        rule_collection.register(test_rule_parent)
        rule_collection.register(test_rule_child)

        template, _ = decode_str('{"key": "value"}')
        runner = Runner(rule_collection, None, template, ["us-east-1"], [])
        failures = runner.run()

        self.assertListEqual(
            failures, [Match(1, 2, 1, 7, None, test_rule_child, "failure")]
        )

    def test_child_rules_suppressed(self):
        class TestRuleParent(CloudFormationLintRule):
            """Def Rule"""

            id = "E1000"
            shortdesc = "Test Rule"
            description = "Test Rule Description"
            source_url = "https://github.com/aws-cloudformation/cfn-python-lint/"
            tags = ["resources"]
            child_rules: Dict[str, Any] = {"E1001": None}

            def match(self, _):
                if self.child_rules.get("E1001"):
                    return self.child_rules["E1001"].failure()
                return []

        class TestRuleChild(CloudFormationLintRule):
            """Def Rule"""

            id = "E1001"
            shortdesc = "Test Rule"
            description = "Test Rule Description"
            source_url = "https://github.com/aws-cloudformation/cfn-python-lint/"
            tags = ["resources"]

            def failure(self):
                return [RuleMatch(["key"], "failure", rule=self)]

        rule_collection = RulesCollection(ignore_rules=["E1001"])
        test_rule_parent = TestRuleParent()
        test_rule_child = TestRuleChild()
        rule_collection.register(test_rule_parent)
        rule_collection.register(test_rule_child)

        template, _ = decode_str('{"key": "value"}')
        runner = Runner(rule_collection, None, template, ["us-east-1"], [])
        failures = runner.run()

        self.assertListEqual(failures, [])

    def test_child_rules_configured(self):
        class TestRuleParent(CloudFormationLintRule):
            """Def Rule"""

            id = "E1000"
            shortdesc = "Test Rule"
            description = "Test Rule Description"
            source_url = "https://github.com/aws-cloudformation/cfn-python-lint/"
            tags = ["resources"]
            child_rules: Dict[str, Any] = {"E1001": None}

            def match(self, _):
                return self.child_rules["E1001"].failure()

        class TestRuleChild(CloudFormationLintRule):
            """Def Rule"""

            id = "E1001"
            shortdesc = "Test Rule"
            description = "Test Rule Description"
            source_url = "https://github.com/aws-cloudformation/cfn-python-lint/"
            tags = ["resources"]

            def __init__(self):
                """Init"""
                super(TestRuleChild, self).__init__()
                self.config_definition = {"pass": {"default": True, "type": "boolean"}}
                self.configure()

            def failure(self):
                if self.config["pass"]:
                    return [RuleMatch(["key"], "failure", rule=self)]
                return []

        rule_collection = RulesCollection()
        test_rule_parent = TestRuleParent()
        test_rule_child = TestRuleChild()
        rule_collection.register(test_rule_parent)
        rule_collection.register(test_rule_child)
        rule_collection.configure(configure_rules={"E1001": {"pass": False}})

        template, _ = decode_str('{"key": "value"}')
        runner = Runner(rule_collection, None, template, ["us-east-1"], [])
        failures = runner.run()

        self.assertListEqual(failures, [])



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

import os
from test.testlib.testcase import BaseTestCase

import cfnlint.core
import cfnlint.helpers  # pylint: disable=E0401


class TestGetMatches(BaseTestCase):
    """Test Get Matches"""

    def test_multiple_templates(self):
        """test multiple templates"""

        filenames = [
            "test/fixtures/templates/bad/noecho.yaml",
            "test/fixtures/templates/bad/issues.yaml",
        ]

        (args, filenames, _) = cfnlint.core.get_args_filenames(filenames)
        matches = cfnlint.core.get_matches(filenames, args)
        self.assertEqual(
            ["E1012", "W4002", "W4002"], [match.rule.id for match in matches]
        )



"""
Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
SPDX-License-Identifier: MIT-0
"""

from test.unit.rules import BaseRuleTestCase

from cfnlint.rules.resources.DependsOn import DependsOn  # pylint: disable=E0401


class TestResourceDependsOn(BaseRuleTestCase):
    """Test base template"""

    def setUp(self):
        """Setup"""
        super(TestResourceDependsOn, self).setUp()
        self.collection.register(DependsOn())

    def test_file_positive(self):
        """Test Positive"""
        self.helper_file_positive()

    def test_file_negative(self):
        """Test failure"""
        self.helper_file_negative(
            "test/fixtures/templates/bad/resources_dependson.yaml", 4
        )
